/*
     This program creates a thread that changes its characteristic
     over time, form "compute bound" to "I/O bound" to blocked.

     Use this program along with various monitoring programs
     to get an idea of how the operating system schedules threads.
*/
#include <math.h>
#include <stdio.h>    // printf()
#include <windows.h>  // win32, Sleep()
#include <pthread.h>

int main()
{
   // thread function prototype
   void *threadWork(void *vargp);

   pthread_t tid;

   // create second thread (note that this function call has no error checking)
   pthread_create(&tid, NULL, threadWork, NULL);
   printf("Created thread.\n");

   // create a third thread
   //pthread_create(&tid, NULL, threadWork, NULL);
   //printf("Created thread.\n");

   while(1) // loop for ever
   {
      Sleep(1000);  // try commenting out this line
   }

   return 0;
}//main


/*
    This thread cycles endlessly between being compute bound,
    being I/O bound, and sleeping (i.e., being blocked).
    The three parameters computeTime, sleepTime, and ioTime
    can be adjusted to control the relative amounts of each
    of these kinds of "work".
*/
void *threadWork(void *vargp)
{
   const double x = 3.14159;
   const double e = 2.7183;
   const int computeTime = 300000000;
   const int sleepTime = 15000;
   const int ioTime = 400000;

   while(1)
   {
      printf("compute\n");
      for(int i = 0; i < computeTime; ++i)
      {
         double y = pow(x, e);
         y = y * y;
      }

      printf("sleep\n");
      Sleep(sleepTime);   // win32

      printf("I/O\n");
      for(int i = 0; i < ioTime; ++i)
      {
         printf("a");
      }
      printf("\n");
   }
   return NULL;
}//threadWork
